package com.bwie.server.service.impl;

import com.alibaba.fastjson.JSON;
import com.bwie.server.utils.DelayedQueue;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.BorrowMoney;
import com.bwie.common.domain.request.BorrowMoneyReq;
import com.bwie.common.domain.response.UserRes;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.server.feign.FeignES;
import com.bwie.server.mapper.BorrowMoneyMapper;
import com.bwie.server.service.BorrowMoneyService;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.concurrent.DelayQueue;
import java.util.concurrent.TimeUnit;

/**
 * Description:
 *
 * @Author: sun-cool-boy
 * @Date: 2023/10/10
 * @info: 业务实现类
 */
@Service
public class BorrowMoneyServiceImpl implements BorrowMoneyService {

    /**
     * 注入借款持久层
     */
    @Resource
    private BorrowMoneyMapper borrowMoneyMapper;

    /**
     * 远程调用鉴权 查询当前用户信息
     */
    @Resource
    private FeignES feignES;

    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    @Resource
    private HttpServletRequest request;

    @Resource
    private  DelayedQueue delayedQueue;

    @Resource
    private RedissonClient redissonClient;


    /**
     * 查询借款信息
     * @Parment borrowMoneyReq
     * @return
     */
    @Override
    public Result<List<BorrowMoney>> selectBorrowMoney(BorrowMoneyReq borrowMoneyReq) {
        UserRes data = getInfo();
        //查询当前用户权限为投资人还是借款人
        Integer roleId = data.getRoleId();
        if (roleId==1){ //借款人 查看自己
            borrowMoneyReq.setUserId(data.getId());
        }
        List<BorrowMoney> borrowMonies = borrowMoneyMapper.selectBorrowMoney(borrowMoneyReq);
        return Result.success(borrowMonies);
    }

    private UserRes getInfo(){
        String userKey = JwtUtils.getUserKey(request.getHeader("token"));
        return JSON.parseObject(stringRedisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey), UserRes.class);
    }

    /**
     * 计算
     * @param borrowMoney
     * @return
     */
    @Override
    public Result<BorrowMoney> calculate(BorrowMoney borrowMoney) {
        //借款金额
        Integer borrowMoney1 = borrowMoney.getBorrowMoney();
        //日利率
        Double interest = borrowMoney.getInterest();
        //月利率 综合利率
        double v = interest * 30;
        //还款期数
        Integer repaymentPlan = borrowMoney.getRepaymentPlan();
        //	手续费计算公式：比如在2022-11-22日借款12000元，分6期还款，按照利率计算为：0.003%
        double one = borrowMoney1 * interest;
        //	计算公式为：
        //	每月还本付息金额=（本金/还款月数）+（本金-累计已还本金）×月利率
        //	每月本金=总本金/还款月数
        //	每月利息=（本金-累计已还本金）×月利率
        double v1 = (borrowMoney1/repaymentPlan) * v + borrowMoney1/repaymentPlan;
        //	还款总额= 贷款额 * 月利率 + 贷款额
        double num = borrowMoney1 * v + borrowMoney1;
        //创建返回对象
        BorrowMoney borrowMoneyRes = new BorrowMoney();;
        //综合利率 /月利率
        borrowMoneyRes.setInterest(v);
        //每月还款
        borrowMoneyRes.setEveryMonthMoney(v1);
        //还款总额
        borrowMoneyRes.setBorrowMoney((int) num);
        //手续费
        borrowMoneyRes.setHandlingCharge(one);
        return Result.success(borrowMoneyRes);
    }

    //开启事务
    @Transactional
    @Override
    public Result addBorrowMoney(BorrowMoney borrowMoney) {
        //执行添加操作
        double v = 1 + borrowMoney.getInterest();
        Integer borrowMoney1 = borrowMoney.getBorrowMoney();
        double v2 = borrowMoney1;
        double money = v2 / v;

        //转为int
        borrowMoney.setBorrowMoney((int) money);
        borrowMoney.setUserId(getInfo().getId());
        //执行添加
        Integer i = borrowMoneyMapper.addBorrowMoney(borrowMoney);
        System.out.println(borrowMoney.getId());
        //同步es
        feignES.addBatch();
        Integer id = borrowMoney.getId();
        //先设置三十秒看效果
        delayedQueue.sendDelayedQueue("one",id.toString(),30000);
        return Result.success();
    }

    @Override
    public Result<List<BorrowMoney>> giveESData() {
        List<BorrowMoney> list = borrowMoneyMapper.giveESData();
        return Result.success(list);
    }

    /**
     * 更改状态
     * @return
     */
    @Override
    public Result upd(Integer id) {
        RLock one = redissonClient.getLock("one");
        try {
            //获取锁
            one.tryLock(2,5, TimeUnit.MINUTES);
            //执行操作
            Integer upd = borrowMoneyMapper.upd(id,1);
            if (upd > 0 ){
                one.unlock();
                //es更新数据
                feignES.addBatch();
                return Result.success();
            }
            one.unlock();
            return Result.error("","已经被投资过了");
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }
}
